unit Unit1;
// DIBUltra Project  / LGPL Source  / CopyRight 1999 Sbastien Lon
// Please contact me at sleon@magic.fr (or search "DIBUltra" in Yahoo, Altavista...)
// Avis aux dveloppeur franais : Je vous encourage  me contacter !

// A new DIBUltra user wrote :

//>    Hi, I was wondering if you could show me just a simple example of
//>how to get the DIBUltra unit to draw just a pixel on the screen.  I
//>tried to look at your examples, and learn from there, but I can't get it
//>to show up on the form (or TImage or whatever).  I create it, set it's
//>pixels etc, and then use drawto but it just doesn't show up.

// Seb response :
//
// DIBUltra is not a Direct To Screen API. (All multitasking actual OS
// will not allows that, without using their own API set)
// It is really useful to draw something "off screen" on a temp layer : a DIBUltra.
// When the job is finished, you must paint the DIBUltra on the screen
// (in a TPaintBox.Canvas for a exemple or even on the TForm.Canvas)
// In the following exemple, I repaint the DIB on the Form1.Canvas.
// For that I :
// 1) Create the DIB when the Form1 is create : OnCreate event.
// 2) Destroy the DIB when the Form1 will be destroyed : OnDestroy event.
// 3) When a repaint message is sent to Form1, I draw the DIB on it (see Form1.OnPaint
//     event) with DrawOn DIB method.
// 4) Each time DIB change, I force repaint for his recipient.
//     ex : after the 200 DIB.LineTo... I force repaint on form with Form1.Repaint.

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DIBUltra, DIBType, StdCtrls;

type
  TForm1 = class(TForm)
    Point: TButton;
    Lines: TButton;
    Clear: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure ClearClick(Sender: TObject);
    procedure PointClick(Sender: TObject);
    procedure LinesClick(Sender: TObject);
  private
    { Dclarations prives }
    procedure WMEraseBkgnd(var m: TWMEraseBkgnd); message WM_ERASEBKGND; // To avoid flicker when repaint
  public
    { Dclarations publiques }
    DIB : TDIBUltra;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  DIB := TDIBUltra.Create( Screen.Width, Screen.Height, DUpf15, nil ); // We must create the DIB
  // Even if you resize to the max of the screen, the DIB is enough large !
  // We could add a DIB.BufferSize(ClientWidth, ClientHeight); at TForm1.OnResize event
  // but Windows badly allocate DIBs and can fail with a repetitive Alloc/Desalloc sequence !
  // It is better to alloc too much but once ! (only if you know, your DIB will be resized.)
  DIB.BrushColor := ClBlack;
  DIB.ClearAll;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  DIB.Free; // We must free the DIB !
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  DIB.DrawOn(ClientRect, Canvas, 0,0);
end;

procedure TForm1.ClearClick(Sender: TObject);
begin
  DIB.ClearAll;
  // We need to show the new aspect of the buffer now : we need to repaint it on the form !
  Repaint; // Go indirectly to TForm1.FormPaint(Sender: TObject);
end;

procedure TForm1.PointClick(Sender: TObject);
var n : Integer;
begin
  For n := 0 to 5000 Do // We put 5000 points
    DIB.PixelsIndex[random(ClientWidth), random(ClientHeight)] := random($7FFF); // <== with random color & location
  // We need to show the new aspect of the buffer now : we need to repaint it on the form !
  Repaint; // Go indirectly to TForm1.FormPaint(Sender: TObject);
end;

procedure TForm1.LinesClick(Sender: TObject);
var n : Integer;
begin
  For n := 0 to 200 Do Begin // We put 200 lines
    DIB.PenColorIndex := random($7FFF); // <== with random color
    DIB.LineTo(random(ClientWidth), random(ClientHeight)); // <== at random locations
  End;
  // We need to show the new aspect of the buffer now : we need to repaint it on the form !
  Repaint; // Go indirectly to TForm1.FormPaint(Sender: TObject);
end;

// From TBitmap example : WinHelp of Delphi (Borland)
// This code avoid flicker. Just to have a better look (not useful for DIBUltra)
// comme toute la fiche va tre dessine, la gestion de ce message
// vite les raffichages inutiles de l'arrire-plan
// qui peuvent provoquer des sautillements.
procedure TForm1.WMEraseBkgnd(var m : TWMEraseBkgnd);
begin
  m.Result := LRESULT(False);
end;

end.
